<?php

declare(strict_types=1);

/**
 * Hoa
 *
 *
 * @license
 *
 * New BSD License
 *
 * Copyright © 2007-2017, Hoa community. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Hoa nor the names of its contributors may be
 *       used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS AND CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

namespace Hoa\Websocket\Test\Unit\Exception;

use Hoa\Test;
use Hoa\Websocket;
use Hoa\Websocket\Exception\CloseError as SUT;

/**
 * Class \Hoa\Websocket\Test\Unit\Exception\CloseError.
 *
 * Test suite of the close error exception.
 *
 * @license    New BSD License
 */
class CloseError extends Test\Unit\Suite
{
    public function case_hoa_websocket_exception(): void
    {
        $this
            ->when($result = new SUT('foo', 0))
            ->then
                ->object($result)
                    ->isInstanceOf(Websocket\Exception::class);
    }

    public function case_set_error_code(): void
    {
        $this
            ->given($exception = new SUT('foo', 0))
            ->when($result = $exception->setErrorCode(42))
            ->then
                ->variable($result)
                    ->isNull();
    }

    public function case_get_error_code(): void
    {
        $this
            ->given(
                $errorCode = 42,
                $exception = new SUT('foo', 0),
                $exception->setErrorCode($errorCode)
            )
            ->when($result = $exception->getErrorCode())
            ->then
                ->integer($result)
                    ->isEqualTo($errorCode);
    }
}
